# This file was auto-generated by Fern from our API Definition.

import typing

PicInstanceSettingsValue = typing.Union[
    str, float, int, typing.List[typing.Optional[typing.Any]], typing.Dict[str, typing.Optional[typing.Any]], bool
]
