# This file was auto-generated by Fern from our API Definition.

import typing
from ....types.parameter_constraint import ParameterConstraint
from ....types.cost_function import CostFunction
from ....types.unformalizable_statement import UnformalizableStatement

Statement = typing.Union[ParameterConstraint, CostFunction, UnformalizableStatement]
