# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import SyncClientWrapper
from .python.client import PythonClient
from ..core.client_wrapper import AsyncClientWrapper
from .python.client import AsyncPythonClient


class CodeExecutionClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.python = PythonClient(client_wrapper=self._client_wrapper)


class AsyncCodeExecutionClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.python = AsyncPythonClient(client_wrapper=self._client_wrapper)
